{ hostRegistry, config, lib, pkgs, modulesPath, ... }:

let
  # soapysdr = with pkgs; soapysdr.override {
  #   extraPackages = [
  #     soapyhackrf
  #   ];
  # };
  soapysdr = pkgs.soapysdr-with-plugins;
in
{
  boot.initrd.availableKernelModules = [ "usbhid" ];
  boot.initrd.kernelModules = [ ];
  boot.kernelModules = [ ];
  boot.extraModulePackages = [ ];

  fileSystems."/" = {
    device = "/dev/disk/by-label/NIXOS_SD";
    fsType = "ext4";
  };

  fileSystems."/boot/firmware" = {
    device = "/dev/disk/by-label/FIRMWARE";
    fsType = "vfat";
  };

  swapDevices = [ ];

  hardware.enableRedistributableFirmware = true;
  #networking.wireless.enable = true;
  hardware.deviceTree.enable = false;
  powerManagement.cpuFreqGovernor = lib.mkDefault "performance";

  boot = {
    loader = {
      grub.enable = false;
      generic-extlinux-compatible.enable = lib.mkDefault true;
      raspberryPi = {
        enable = true;
        version = 4;
        uboot.enable = true;
        firmwareConfig = ''
          gpu_mem=192
          dtparam=audio=on
        '';
      };
    };
    kernelPackages = pkgs.linuxPackages_latest;
    # Don't build ZFS for aarch64 (broken?)
    supportedFilesystems = lib.mkForce [ "vfat" "ext4" ];

    tmpOnTmpfs = true;
  };
  sdImage = {
    compressImage = false;
    imageBaseName = config.networking.hostName;
    firmwareSize = 512;
  };

  c3d2 = {
    isInHq = false;
    hq.statistics.enable = true;
    pi-sensors = [ {
      type = "dht22";
      pin = 17;
      location = "Schrank";
    } {
      type = "dht22";
      pin = 23;
      location = "Aussen";
    } ];
  };

  nix = {
    buildCores = 4;
    maxJobs = 2;
  };

  networking = {
    hostName = "radiobert"; # Define your hostname.
    useDHCP = false;
    interfaces.eth0.ipv4.addresses = [{
      address = hostRegistry.hosts.${config.networking.hostName}.ip4;
      prefixLength = 26;
    }];
    defaultGateway = "172.20.73.1";
    firewall.enable = false;
    nameservers = [ "172.20.73.8" "9.9.9.9" ];
  };

  hardware.bladeRF.enable = true;
  users.users.soapysdr = {
    isSystemUser = true;
    group = "soapysdr";
  };
  users.groups.soapysdr = {};
  systemd.services.soapysdr-server = {
    wantedBy = [ "multi-user.target" ];
    serviceConfig = {
      ExecStart = "${soapysdr}/bin/SoapySDRServer --bind";
      User = "soapysdr";
      Group = "soapysdr";
      ProtectSystem = "full";
      ProtectHome = true;
    };
  };

  systemd.services.bladerf-fpga-bitstream = {
    wantedBy = [ "multi-user.target" ];
    serviceConfig = {
      ExecStart =
        let
          # bitstream = pkgs.fetchurl {
          #   url = "https://www.nuand.com/fpga/hostedx40-latest.rbf";
          #   sha256 = "0fx95rm4004hfxc1708sf35m342iksdhcqiffgxhgkqjd6rxjpz1";
          # };

          # Matches libbladeRF-2.2.1
          bitstream = pkgs.fetchurl {
            url = "https://www.nuand.com/fpga/v0.10.2/hostedx40.rbf";
            sha256 = "091qiyw971wwbl1z49n8miqmjk4h5m8bv4nyc5zf77j5jjl74j5b";
          };

        in ''${pkgs.libbladeRF}/bin/bladeRF-cli -d "libusb: instance=0" -l ${bitstream}'';
      RemainAfterExit = true;
    };
  };

  services.collectd.plugins = {
    curl_json = ''
      <URL "http://localhost:8073/metrics.json">
        Plugin "openwebrx_decodes"
        <Key "wsjt/decodes/*/*/count">
          Type "counter"
        </Key>
        <Key "aprs/decodes/*/*/*/count">
          Type "counter"
        </Key>
        <Key "js8call/decodes/*/*/count">
          Type "counter"
        </Key>
        <Key "decoding/queue/*/count">
          Type "counter"
        </Key>
        <Key "decoding/queue/length">
          Type "gauge"
        </Key>
        <Key "pskreporter/*/count">
          Type "gauge"
        </Key>
        <Key "wsprnet/*/count">
          Type "gauge"
        </Key>
        <Key "openwebrx/users">
          Type "gauge"
        </Key>
      </URL>
    '';
  };

  environment.systemPackages = with pkgs; [
    libraspberrypi
    raspberrypi-eeprom
    vim
    wget
    dump1090_sdrplus
    soapysdr-with-plugins
    libbladeRF
  ];

  programs.tmux.enable = true;

  # Do not log to flash:
  services.journald.extraConfig = ''
    Storage=volatile
  '';

  # Enable the OpenSSH daemon.
  services.openssh = {
    enable = true;
  };

  # Allow access to USB
  services.udev.extraRules = ''
    SUBSYSTEM=="usb", MODE:="0666"
  '';

  # This value determines the NixOS release from which the default
  # settings for stateful data, like file locations and database versions
  # on your system were taken. It‘s perfectly fine and recommended to leave
  # this value at the release version of the first install of this system.
  # Before changing this value read the documentation for this option
  # (e.g. man configuration.nix or on https://nixos.org/nixos/options.html).
  system.stateVersion = "21.05"; # Did you read the comment?
}
